from concurrent.futures import Future
from functools import wraps
from subprocess import Popen
from typing import Callable, Union

from ..util.type import T


def run_blocking(func: Callable[..., Union[Future, Popen]]) -> Callable[..., T]:

    @wraps(func)
    def wrapper(*args, **kwargs) -> T:
        task = func(*args, **kwargs)
        if isinstance(task, Future):
            return task.result()
        elif isinstance(task, Popen):
            task.wait()
            return task.returncode
        else:
            raise TypeError("run_blocking only works with Future or Popen")

    return wrapper




from .audio import AudioWriter
from .image import ImageWriter
from .json import JsonWriter
from .matlab import MatWriter
from .pickle import PickleWriter
from .text import TextWriter
from .video import VideoWriter


class Writer:

    def __init__(self):
        self.image = ImageWriter
        self.text = TextWriter
        self.json = JsonWriter
        self.video = VideoWriter
        self.audio = AudioWriter
        self.mat = MatWriter
        self.pickle = PickleWriter



import subprocess

ffmpeg_available = subprocess.run(['ffmpeg', '-version'], stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL,
                                  shell=True).returncode == 0

__all__ = ['ffmpeg_available']

if ffmpeg_available:
    from .ffmpeg import ffmpeg_command, video2frames, merge_video_audio, resample_video_fps, mp32wav
    __all__ += ['ffmpeg_command', 'video2frames', 'merge_video_audio', 'resample_video_fps', 'mp32wav']



from .audio import AudioReader
from .image import ImageReader
from .json import JsonReader
from .matlab import MatReader
from .pickle import PickleReader
from .text import TextReader
from .video import VideoReader


class Reader:

    def __init__(self):
        self.image = ImageReader
        self.text = TextReader
        self.json = JsonReader
        self.video = VideoReader
        self.audio = AudioReader
        self.mat = MatReader
        self.pickle = PickleReader
